// C++ example using LoadLibrary and common DLL interface
// (provided for reference for those who will use spellchecker from other languages). 
// It is better to use C++ interface.

#include <Windows.h>
#include <Winbase.h>
#include <stdio.h>

// typedefs for imported functions
typedef const char* (__stdcall *FNGetLastSpellCheckerError)();
typedef unsigned int (__stdcall *FNLoadSpellCheckerDB)(const char* pszPath);
typedef void (__stdcall *FNCloseSpellDBHandle)(unsigned int hDB);
typedef unsigned int (__stdcall *FNCreateSpellChecker)(unsigned int hDB, int fReset);
typedef void (__stdcall *FNCloseSpellHandle)(unsigned int hSP);
typedef bool (__stdcall *FNGetSpellCheckerVersion)(char* pszBuffer, int nBufferSize, const char *pszPathLDB);
typedef void (__stdcall *FNSetSpellCheckerLicensePath)(const char* pszFileName);
// Spellchecking
typedef unsigned int (__stdcall *FNCheckWord)(unsigned int hSP, const char* pszWord, const char* plszLang);
//typedef unsigned int (__stdcall *FNCheckWordWithLang)(unsigned int hSP, const char* pszWord, const char* pszLang);
typedef void (__stdcall *FNEnableAllLang)(unsigned int hSP, bool eFlag);
typedef int (__stdcall *FNGetResultType)(unsigned int hSR);
typedef int (__stdcall *FNGetCandidateCount)(unsigned int hSR);
typedef const char* (__stdcall *FNGetCandidateWord)(unsigned int hSR, int nIndex);
typedef const char* (__stdcall *FNGetCandidateLanguage)(unsigned int hSR);
typedef int (__stdcall *FNGetCandidateWeight)(unsigned int hSR, int nIndex);
typedef void (__stdcall *FNCloseResultHandle)(unsigned int hSR);
// Settings
typedef void (__stdcall *FNSetMaxError)(unsigned int hSP, int eTune, int nBound);
typedef int (__stdcall *FNGetMaxError)(unsigned int hSP, int* pnBound);
typedef void (__stdcall *FNSetMinProbability)(unsigned int hSP, int eTune, int nWeight);
typedef int (__stdcall *FNGetMinProbability)(unsigned int hSP, int* pnWeight);
typedef void (__stdcall *FNSetSpellFilter)(unsigned int hSP, int eSkip);
typedef int (__stdcall *FNGetSpellFilter)(unsigned int hSP);
typedef void (__stdcall *FNResetDB)(unsigned int hSP, unsigned int hDB, int fReset);

int main()
{
	// load library and import functions
	HINSTANCE hDLL = LoadLibrary("SpellCheckerLib");
	if (hDLL == NULL)
		return -1;

	FNGetLastSpellCheckerError GetLastSpellCheckerError = (FNGetLastSpellCheckerError)GetProcAddress(hDLL, "GetLastSpellCheckerError");
	FNSetSpellCheckerLicensePath SetSpellCheckerLicensePath = (FNSetSpellCheckerLicensePath)GetProcAddress(hDLL, "SetSpellCheckerLicensePath");
	FNLoadSpellCheckerDB LoadSpellCheckerDB = (FNLoadSpellCheckerDB)GetProcAddress(hDLL, "LoadSpellCheckerDB");
	FNCloseSpellDBHandle CloseSpellDBHandle = (FNCloseSpellDBHandle)GetProcAddress(hDLL, "CloseSpellDBHandle");
	FNCreateSpellChecker CreateSpellChecker = (FNCreateSpellChecker)GetProcAddress(hDLL, "CreateSpellChecker");
	FNCloseSpellHandle CloseSpellHandle = (FNCloseSpellHandle)GetProcAddress(hDLL, "CloseSpellHandle");
	FNSetSpellFilter SetSpellFilter = (FNSetSpellFilter)GetProcAddress(hDLL, "SetSpellFilter");
	
	FNCheckWord CheckWord = (FNCheckWord)GetProcAddress(hDLL, "CheckWord");
	//FNCheckWordWithLang CheckWordWithLang = (FNCheckWordWithLang)GetProcAddress(hDLL, "CheckWordWithLang");
	FNEnableAllLang EnableAllLangProcessing = (FNEnableAllLang)GetProcAddress(hDLL, "EnableAllLanguagesProcessing");
	FNGetResultType GetResultType = (FNGetResultType)GetProcAddress(hDLL, "GetResultType");
	FNGetCandidateCount GetCandidateCount = (FNGetCandidateCount)GetProcAddress(hDLL, "GetCandidateCount");
	FNGetCandidateWord GetCandidateWord = (FNGetCandidateWord)GetProcAddress(hDLL, "GetCandidateWord");
	FNGetCandidateLanguage GetCandidateLanguage = (FNGetCandidateLanguage)GetProcAddress(hDLL, "GetCandidateLanguage");
	FNGetCandidateWeight GetCandidateWeight = (FNGetCandidateWeight)GetProcAddress(hDLL, "GetCandidateWeight");
	FNCloseResultHandle CloseResultHandle = (FNCloseResultHandle)GetProcAddress(hDLL, "CloseResultHandle");

	// Step1: set license
	const char* pszLicensePath = "ISDK_License.xml";
	SetSpellCheckerLicensePath(pszLicensePath);
	
	// Step2: create DB
	const char* pszLDBPath = "LDB";
	unsigned int hDB = LoadSpellCheckerDB(pszLDBPath);
	if (hDB == 0)
	{
		printf("Cannot load LDB\n");
		printf(GetLastSpellCheckerError());
		return -1;
	}

	// Step3: create instance of spellchecker
	unsigned int hSpeller = CreateSpellChecker(hDB, 0);
	if (hSpeller == 0)
	{
		printf("Cannot create spellchecker");
		CloseSpellDBHandle(hDB);
		return 2;
	}

	// Step4: set options
	int fSkipFlag = 0;
	SetSpellFilter(hSpeller, fSkipFlag);

	// Step4: anables(true) or disables(false) spellchecking with all languages... may be skiped
	EnableAllLangProcessing(hSpeller, false);

	// Step5.1: spellchecking... may be repeated many times
	const char* pszWord = "devicez";
	unsigned int hResult = CheckWord(hSpeller, pszWord, 0);
	if (hResult != 0)
	{
		int eVar = GetResultType(hResult);
		const int nSize = GetCandidateCount(hResult);
		printf(pszWord);
		printf("\t");
		switch(eVar) 
		{
		case 0:
			printf("skip\n");
			break;
		case 1:
			printf("correct\n");
			break;
		case 2:
			printf("incorrect\n");
			break;
		case 3:
			printf("bad\n");
			break;
		}
		const char* pszCandidate;
		int	nWeight;

		for (int nIndex = 0; nSize != nIndex; ++nIndex)
		{
			pszCandidate = GetCandidateWord(hResult, nIndex);
			nWeight = GetCandidateWeight(hResult, nIndex);
			if (nWeight != 100) // 100 is the weight of original word if it is correct
			{
				printf("%s\t%d\n", pszCandidate, nWeight);
			}
		}
		// close result handle after results are processed
		CloseResultHandle(hResult);
	}

	// Step5.2: spellchecking with specified language... may be repeated many times
	pszWord = "sotfware";
	const char* pszLanguage = "ENGLISH";
	hResult = CheckWord(hSpeller, pszWord, pszLanguage);
	if (hResult != 0)
	{
		int eVar = GetResultType(hResult);
		const int nSize = GetCandidateCount(hResult);
		printf(pszWord);
		printf("\t");
		switch(eVar) 
		{
		case 0:
			printf("skip\n");
			break;
		case 1:
			printf("correct\n");
			break;
		case 2:
			printf("incorrect\n");
			break;
		case 3:
			printf("bad\n");
			break;
		}
		const char* pszCandidate;
		int	nWeight;

		for (int nIndex = 0; nSize != nIndex; ++nIndex)
		{
			pszCandidate = GetCandidateWord(hResult, nIndex);
			nWeight = GetCandidateWeight(hResult, nIndex);
			if (nWeight != 100) // 100 is the weight of original word if it is correct
			{
				printf("%s\t%d\n", pszCandidate, nWeight);
			}
		}

		// print word's language
		const char* sLang = GetCandidateLanguage(hResult);
		printf("Word %s was checked in %s language", pszWord, sLang);

		// close result handle after results are processed
		CloseResultHandle(hResult);
	}

	// Step5: close handles
	CloseSpellHandle(hSpeller);
	CloseSpellDBHandle(hDB);
	
	return 0;
}