// File: CSharp-Example.cs
// Description: Spell checker sample.
// Copyright: (C) 2006 EffectiveSoft Ltd. All Rights Reserved.
// Technical support: technical-support@effective-soft.com

using System;
using EffectiveSoft.Linguistics;

namespace CSharp_Example
{
	/// <summary>
	/// Spellchecker sample application.
	/// </summary>
	class Application
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			string[] textResult = {"skip", "correct", "incorrect", "bad"};
			string LDBPath = "LDB";
			string word = "Computre";
            string language = "ENGLISH";
			SpellChecker.EResult result;
			int size;

			try
			{
				if (args.Length == 1)
					LDBPath = args[0];
				// provide path to license file
				SpellLicense.SetPath("ISDK_License.xml");
				// Load database that is required to create the instance of SpellChecker.
				using (SpellLDB ldb = new SpellLDB(LDBPath))
				{
					// Create of SpellChecker class instance
					SpellChecker spellChecker = new SpellChecker(ldb);

					// Check word
					result = spellChecker.CheckWord(word);
					Console.WriteLine("{0}\t{1}", word, textResult[(int)result]);

					// Get number of candidates
					size = spellChecker.GetCandidateCount();

					// Get and print candidates
					for (int i = 0; i < size; ++i)
						Console.WriteLine("\t{0}\t{1}", 
							spellChecker.GetCandidate(i).Candidate, spellChecker.GetCandidate(i).Weight);

                    spellChecker.AllLanguageProcessing = true;
                    result = spellChecker.CheckWord(word, language);
                    Console.WriteLine("{0}\t{1}", word, textResult[(int)result]);
                    size = spellChecker.GetCandidateCount();
                    for (int i = 0; i < size; ++i)
                        Console.WriteLine("\t{0}\t{1}",
                            spellChecker.GetCandidate(i).Candidate, spellChecker.GetCandidate(i).Weight);

                    spellChecker.AllLanguageProcessing = false;
                    // changing skip word rules 
					spellChecker.SpellFilter = SpellChecker.ESkipWord.ESkipHasUpper | spellChecker.SpellFilter;

					result = spellChecker.CheckWord(word);
					Console.WriteLine("{0}\t{1}", word, textResult[(int)result]);
				}
			}
				// Let the user know what went wrong.
			catch (SpellException e) 
			{
				Console.WriteLine("Spellchecker Exception: {0}.", e.Message);
			}
			catch (Exception e) 
			{
				Console.WriteLine("Exception: {0}", e.Message);
				Console.WriteLine("{0}", e.StackTrace);
			}
		}
	}
}
